#!/bin/bash
#
# Run gnulib tests under Syd.
#
# Copyright 2024 Ali Polatel <alip@chesswob.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

# TEST_LIMIT: Number of tests to run at a time.
# TEST_PATTERN: A Regex (PCRE) for tests to run.
case "$1" in
    '')
        TEST_LIMIT=250
        TEST_PATTERN='.*'
        ;;
    [0-9]*)
        TEST_LIMIT="${1}"
        TEST_PATTERN='.*'
        ;;
    *)
        TEST_LIMIT=250
        TEST_PATTERN="${1}"
        ;;
esac

# A Regex (PCRE) for tests to skip.
SKIP_PATTERN='(bison|datetime|exception|link-warning|printf|trapping|windows|^(array|java|uni)|-ieee$|(^(execinfo|gnumakefile|havelib|timevar)$))'

# Make sure we don't trigger TPE.
umask 077

# Disable coredumps.
ulimit -c 0

SYD="${CARGO_BIN_EXE_syd:-syd}"

set -ex
DIR="$(mktemp -d --tmpdir=/tmp syd-gnulib.XXXXX)"
set +ex

function finish() {
    rm -rf "${DIR}"
}

trap finish EXIT

edo() {
    echo >&2 "$*"
    "$@"
}

elog() {
    echo >&2 "$*"
    "$@" >.log 2>&1
    r=$?
    [[ $r == 0 ]] || cat .log
    rm -f .log
    return $r
}

run_test() {
    local name="$1"
    local tdir="$(readlink -f "${DIR}")"
    local tnam="$(echo "${name}" | sed -e 's|/|-|g')"
    elog "${SYD}" -ppaludis \
        ./gnulib-tool \
            --quiet \
            --dir "${tdir}/${tnam}" \
            --test "${name}"
    r=$?
    if [[ $r == 0 ]]; then
        rm -fr "${tdir}/${tnam}"
    else
        echo "Keeping test directory of failed test: ${tdir}/${tnam}"
    fi
    return $r
}

set -ex
pushd "${DIR}"
git clone --depth 1 https://git.savannah.gnu.org/git/gnulib.git
pushd gnulib
sed -i \
    -e 's|"0.0.0.0"|"127.0.0.1"|' \
    tests/test-getsockname.c
set +ex

PASS=0
FAIL=0
SKIP=0
CTEST=$(./gnulib-tool --list | grep -P "${TEST_PATTERN}" | grep -vP "${SKIP_PATTERN}" | wc -l)
NTEST=${TEST_LIMIT}
if [[ ${NTEST} -gt ${CTEST} ]]; then
    NTEST=${CTEST}
fi

idx=0
while read -r -d$'\n' name; do
    : $(( idx++ ))
    echo -e "\033[92m*** $name ($idx of $NTEST: $PASS ok, $FAIL notok, $SKIP todo) ***\033[0m"
    if echo "${name}" | grep -qP "${SKIP_PATTERN}"; then
        echo "ok ${idx} - ${name} # TODO"
        : $(( SKIP++ ))
    elif run_test "${name}"; then
        echo "ok ${idx} - ${name}"
        : $(( PASS++ ))
    else
        echo "not ok ${idx} - ${name} - FAIL: $?"
        : $(( FAIL++ ))
    fi
done < <(./gnulib-tool --list | grep -P "${TEST_PATTERN}" | grep -vP "${SKIP_PATTERN}" | shuf | head -n"${NTEST}")

echo "# $PASS tests passed."
echo "# $FAIL tests failed."
echo "# $SKIP tests skipped."
exit $FAIL
