//
// syd: seccomp and landlock based application sandbox with support for namespaces
// src/config.rs: Static configuration, edit & recompile!
//
// Copyright (c) 2023, 2024 Ali Polatel <alip@chesswob.org>
// Based in part upon HardenedBSD's sys/hardenedbsd/hbsd_pax_segvguard.c which is:
//   Copyright (c) 2006 Elad Efrat <elad@NetBSD.org>
//   Copyright (c) 2013-2017, by Oliver Pinter <oliver.pinter@hardenedbsd.org>
//   Copyright (c) 2014, by Shawn Webb <shawn.webb@hardenedbsd.org>
//   Copyright (c) 2014, by Danilo Egea Gondolfo <danilo at FreeBSD.org>
//   All rights reserved.
//   SPDX-License-Identifier: BSD-3-Clause
// Based in part upon gVisor's kvm_const*.go which is:
//   Copyright 2018 The gVisor Authors.
//   SPDX-License-Identifier: Apache-2.0
// Based in part upon kvm-ioctls' kvm-ioctls.rs which is:
//   Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
//   SPDX-License-Identifier: Apache-2.0 OR MIT
//
// SPDX-License-Identifier: GPL-3.0-or-later

use std::time::Duration;

use nix::unistd::{sysconf, SysconfVar};
use once_cell::sync::Lazy;

use crate::{caps::Capability, landlock::ABI};

// Convenience type to list {io,pr}ctls with their names.
type KeyValue<'a> = (&'a str, u64);

/// Landlock API in use.
pub const LANDLOCK_ABI: ABI = ABI::V3;
/// Landlock API string representation.
pub const LANDLOCK_ABI_STR: &str = "v3";

/// Path to /etc.
pub const PATH_ETC: &str = "/etc";

/// Path to a temporary directory.
/// This directory is used to create temporary directories during
/// private shm & tmp creation so for the cleanup at exit to work
/// this directory must not be under /dev/shm or /tmp.
pub const PATH_TMP: &str = "/var/tmp";

/// The default shell to execute.
/// Change this if your system doesn't have /bin/sh,
/// or set the environment variable SYD_SH.
pub const SYD_SH: &str = "/bin/sh";

/// The environment variable to read the default shell from
pub const ENV_SH: &str = "SYD_SH";
/// The environment variable to read the log level from.
pub const ENV_LOG: &str = "SYD_LOG";

/// The environment variable to set to disable logging to syslog.
pub const ENV_NO_SYSLOG: &str = "SYD_NO_SYSLOG";

/// The environment variable to set to disable cross memory attach, and fallback to /proc/pid/mem.
pub const ENV_NO_CROSS_MEMORY_ATTACH: &str = "SYD_NO_CROSS_MEMORY_ATTACH";

/// The environment variable to set to log to a different fd than standard error.
pub const ENV_LOG_FD: &str = "SYD_LOG_FD";

/// Specify the duration for which additional threads outside the core
/// pool remain alive while not receiving any work before giving up and
/// terminating.
pub const MON_KEEPALIVE: Duration = Duration::from_secs(90);

/// Path prefix for magic stat commands.
pub const MAGIC_PREFIX: &str = "/dev/syd";

/// Path to the file which will be used as file status for magic stat commands.
pub const MAGIC_FILE: &str = "/dev/null";

/// syd version
pub static VERSION: Lazy<&'static str> = Lazy::new(|| {
    #[allow(clippy::if_same_then_else)]
    #[allow(clippy::branches_sharing_code)]
    if env!("SYD_GITHEAD").is_empty() {
        env!("CARGO_PKG_VERSION")
    } else {
        env!("SYD_GITHEAD")
    }
});

/// Api version of the configuration.
pub const API_VERSION: &str = env!("CARGO_PKG_VERSION_MAJOR");

/// System page size
#[allow(clippy::cast_sign_loss)]
pub static PAGE_SIZE: Lazy<u64> = Lazy::new(|| {
    sysconf(SysconfVar::PAGE_SIZE)
        .unwrap_or(Some(4096))
        .unwrap_or(4096) as u64
});

/// Default buffer size used in `compat::get_directory_entries` function.
pub const GETDENTS_BUFFER_SIZE: usize = 8192;

/// Max entries to be allocated by LDD vector.
pub const LDD_MAX_ENTRIES: usize = 32;

// SegvGuard Defaults

/// SegvGuard entry expiry timeout.
pub const SEGVGUARD_EXPIRY: Duration = Duration::from_secs(2 * 60);
/// SegvGuard entry suspension timeout.
pub const SEGVGUARD_SUSPENSION: Duration = Duration::from_secs(10 * 60);
/// SegvGuard max number of crashes before expiry.
pub const SEGVGUARD_MAXCRASHES: u8 = 5;

// Sandbox Restrictions

/// List of allowlisted prctl requests.
pub const ALLOWLIST_PRCTL: &[KeyValue] = &[
    ("PR_SET_PDEATHSIG", 1),
    ("PR_GET_PDEATHSIG", 2),
    ("PR_GET_DUMPABLE", 3),
    ("PR_SET_DUMPABLE", 4),
    ("PR_GET_ALIGN", 5),
    ("PR_GET_SETALIGN", 6),
    ("PR_GET_KEEPCAPS", 7),
    ("PR_SET_KEEPCAPS", 8),
    ("PR_GET_FPEMU", 9),
    ("PR_SET_FPEMU", 10),
    ("PR_GET_FPEXC", 11),
    ("PR_SET_FPEXC", 12),
    ("PR_GET_TIMING", 13),
    ("PR_SET_TIMING", 14),
    ("PR_SET_NAME", 15),
    ("PR_GET_NAME", 16),
    ("PR_GET_ENDIAN", 19),
    ("PR_SET_ENDIAN", 20),
    ("PR_GET_SECCOMP", 21),
    ("PR_SET_SECCOMP", 22),
    ("PR_CAPBSET_READ", 23),
    ("PR_CAPBSET_DROP", 24),
    ("PR_GET_TSC", 25),
    ("PR_SET_TSC", 26),
    ("PR_GET_SECUREBITS", 27),
    ("PR_SET_SECUREBITS", 28),
    ("PR_SET_TIMERSLACK", 29),
    ("PR_GET_TIMERSLACK", 30),
    // SAFETY: no perf! PR_TASK_PERF_EVENTS_DISABLE
    // SAFETY: no perf! PR_TASK_PERF_EVENTS_ENABLE
    ("PR_MCE_KILL", 33),
    ("PR_MCE_KILL_GET", 34),
    // SAFETY: no self modifying executables! PR_SET_MM
    // SAFETY: no ptrace! PR_SET_PTRACER
    ("PR_SET_CHILD_SUBREAPER", 36),
    ("PR_GET_CHILD_SUBREAPER", 37),
    ("PR_SET_NO_NEW_PRIVS", 38),
    ("PR_GET_NO_NEW_PRIVS", 39),
    ("PR_GET_TID_ADDRESS", 40),
    ("PR_SET_THP_DISABLE", 41),
    ("PR_GET_THP_DISABLE", 42),
    // SAFETY: unimplemented! PR_MPX_ENABLE_MANAGEMENT
    // SAFETY: unimplemented! PR_MPX_DISABLE_MANAGEMENT
    ("PR_SET_FP_MODE", 45),
    ("PR_GET_FP_MODE", 46),
    ("PR_CAP_AMBIENT", 47),
    ("PR_SVE_SET_VL", 50),
    ("PR_SVE_GET_VL", 51),
    // SAFETY: do not access speculation misfeature! PR_GET_SPECULATION_CTRL
    // SAFETY: do not change speculation misfeature! PR_SET_SPECULATION_CTRL
    ("PR_PAC_RESET_KEYS", 54),
    ("PR_SET_TAGGED_ADDR_CTRL", 55),
    ("PR_GET_TAGGED_ADDR_CTRL", 56),
    ("PR_SET_IO_FLUSHER", 57),
    ("PR_GET_IO_FLUSHER", 58),
    ("PR_SET_SYSCALL_USER_DISPATCH", 59),
    ("PR_PAC_SET_ENABLED_KEYS", 60),
    ("PR_PAC_GET_ENABLED_KEYS", 61),
    ("PR_SCHED_CORE", 62),
    ("PR_SET_MDWE", 65),
    ("PR_GET_MDWE", 66),
    ("PR_SET_VMA", 0x53564d41),
];

/// The list of Linux capabilities(7) to drop on startup.
pub const CAPS_DROP: &[Capability] = &[
    Capability::CAP_AUDIT_CONTROL,
    Capability::CAP_AUDIT_READ,
    Capability::CAP_AUDIT_WRITE,
    Capability::CAP_BLOCK_SUSPEND,
    Capability::CAP_BPF,
    Capability::CAP_CHECKPOINT_RESTORE,
    Capability::CAP_CHOWN,
    Capability::CAP_DAC_OVERRIDE,
    Capability::CAP_DAC_READ_SEARCH,
    Capability::CAP_FSETID,
    Capability::CAP_FOWNER,
    Capability::CAP_IPC_LOCK,
    Capability::CAP_IPC_OWNER,
    Capability::CAP_KILL,
    Capability::CAP_LEASE,
    Capability::CAP_LINUX_IMMUTABLE,
    Capability::CAP_MAC_ADMIN,
    Capability::CAP_MAC_OVERRIDE,
    Capability::CAP_MKNOD,
    Capability::CAP_NET_ADMIN,
    Capability::CAP_NET_BROADCAST,
    Capability::CAP_NET_RAW,
    Capability::CAP_PERFMON,
    Capability::CAP_SETGID,
    Capability::CAP_SETUID,
    Capability::CAP_SETFCAP,
    Capability::CAP_SETPCAP,
    Capability::CAP_NET_BIND_SERVICE,
    Capability::CAP_SYSLOG,
    Capability::CAP_SYS_ADMIN,
    Capability::CAP_SYS_BOOT,
    Capability::CAP_SYS_CHROOT,
    Capability::CAP_SYS_MODULE,
    Capability::CAP_SYS_NICE,
    Capability::CAP_SYS_PACCT,
    Capability::CAP_SYS_PTRACE,
    Capability::CAP_SYS_RAWIO,
    Capability::CAP_SYS_RESOURCE,
    Capability::CAP_SYS_TIME,
    Capability::CAP_SYS_TTY_CONFIG,
    Capability::CAP_WAKE_ALARM,
];

/// Equivalent to the list of environment variables that gl*bc removes for SUID programs.
pub const UNSAFE_ENV: &[&str] = &[
    "GCONV_PATH",
    "GETCONF_DIR",
    "GLIBC_TUNABLES",
    "HOSTALIASES",
    "LD_AOUT_LIBRARY_PATH",
    "LD_AOUT_PRELOAD",
    "LD_AUDIT",
    "LD_BIND_NOT",
    "LD_DEBUG",
    "LD_DEBUG",
    "LD_DEBUG_OUTPUT",
    "LD_DYNAMIC_WEAK",
    "LD_DYNAMIC_WEAK",
    "LD_ELF_HINTS_PATH",
    "LD_HWCAP_MASK",
    "LD_LIBMAP",
    "LD_LIBMAP_DISABLE",
    "LD_LIBRARY_PATH",
    "LD_LIBRARY_PATH_FDS",
    "LD_LIBRARY_PATH_RPATH",
    "LD_LOADFLTR",
    "LD_ORIGIN_PATH",
    "LD_PREFER_MAP_32BIT_EXEC",
    "LD_PRELOAD",
    "LD_PRELOAD_FDS",
    "LD_PROFILE",
    "LD_SHOW_AUXV",
    "LD_USE_LOAD_BIAS",
    "LOCALDOMAIN",
    "LOCPATH",
    "MALLOC_TRACE",
    "NIS_PATH",
    "NLSPATH",
    "RESOLV_HOST_CONF",
    "RES_OPTIONS",
    "TMPDIR",
    "TZDIR",
];

// Sandboxing profiles

/// Libsyd helper sandboxing profile.
/// Turns all sandboxing off and set sandbox lock to Exec.
/// Useful to configure syd in the application via libsyd.
pub const PROFILE_LIB: &[&str] = &[
    "lock:exec",
    "sandbox/read,stat,write,exec,net,force,tpe,mem,pid:off",
];

/// Silent sandboxing profile.
pub const PROFILE_SILENT: &[&str] = &[
    "filter/mem:1",
    "filter/pid:1",
    "filter/tpe:1",
    "filter/read,stat,exec,write+/***",
    "filter/net/bind+/***",
    "filter/net/bind+any!0-65535",
    "filter/net/connect+/***",
    "filter/net/connect+any!0-65535",
];

/// No ipv4 sandboxing profile.
pub const PROFILE_NOIPV4: &[&str] = &[
    "sandbox/net:on",
    "allow/net/connect+any6!0-65535",
    "deny/net/connect+any4!0-65535",
];

/// No ipv6 sandboxing profile.
pub const PROFILE_NOIPV6: &[&str] = &[
    "sandbox/net:on",
    "allow/net/connect+any4!0-65535",
    "deny/net/connect+any6!0-65535",
];

/// Privileged sandboxing profile.
pub const PROFILE_PRIVILEGED: &[&str] = &["trace/allow_unsafe_caps:1"];

/// Container sandboxing profile, activated with:
/// `syd --profile container`
pub const PROFILE_CONTAINER: &[&str] = &[
    "unshare/user,mount,uts,ipc,pid,net,cgroup:1",
    "deny/read,stat,write+/proc/1/**",
    "allow/read,stat+/proc/1/comm",
];

/// Immutable container sandboxing profile, activated with:
/// `syd --profile immutable`
pub const PROFILE_IMMUTABLE: &[&str] = &[
    "unshare/user,mount,uts,ipc,pid,net,cgroup:1",
    "bind+tmpfs:/dev/shm:nodev,nosuid,noexec,mode=1777",
    "bind+tmpfs:/tmp:nodev,nosuid,mode=1777",
    "bind+/etc:/etc:ro,nodev,noexec,nosuid",
    "bind+/home:/home:ro,nodev,noexec,nosuid",
    "bind+/media:/media:ro,nodev,noexec,nosuid",
    "bind+/mnt:/mnt:ro,nodev,noexec,nosuid",
    "bind+/opt:/opt:ro,nodev,nosuid",
    "bind+/srv:/srv:ro,nodev,noexec,nosuid",
    "bind+/usr:/usr:ro,nodev",
    "bind+/etc/shells:/proc/kcore:ro,nodev,noexec,nosuid",
    "bind+/etc/shells:/proc/keys:ro,nodev,noexec,nosuid",
    "bind+/etc/shells:/proc/latency_stats:ro,nodev,noexec,nosuid",
    "bind+/etc/shells:/proc/sysrq-trigger:ro,nodev,noexec,nosuid",
    "bind+/etc/shells:/proc/timer_list:ro,nodev,noexec,nosuid",
    "bind+/etc/shells:/proc/timer_stats:ro,nodev,noexec,nosuid",
    "bind+/var/empty:/proc/acpi:ro,nodev,noexec,nosuid",
    "bind+/var/empty:/proc/asound:ro,nodev,noexec,nosuid",
    "bind+/var/empty:/proc/bus:ro,nodev,noexec,nosuid",
    "bind+/var/empty:/proc/driver:ro,nodev,noexec,nosuid",
    "bind+/var/empty:/proc/dynamic_debug:ro,nodev,noexec,nosuid",
    "bind+/var/empty:/proc/fs:ro,nodev,noexec,nosuid",
    "bind+/var/empty:/proc/irq:ro,nodev,noexec,nosuid",
    "bind+/var/empty:/proc/pressure:ro,nodev,noexec,nosuid",
    "bind+/var/empty:/proc/scsi:ro,nodev,noexec,nosuid",
    "bind+/var/empty:/proc/sys/debug:ro,nodev,noexec,nosuid",
    "bind+/var/empty:/proc/sys/dev:ro,nodev,noexec,nosuid",
    "bind+/proc/sys/fs:/proc/sys/fs:ro,nodev,noexec,nosuid",
    "bind+/proc/sys/kernel:/proc/sys/kernel:ro,nodev,noexec,nosuid",
    "bind+/proc/sys/vm:/proc/sys/vm:ro,nodev,noexec,nosuid",
    "bind+/var/empty:/proc/sysvipc:ro,nodev,noexec,nosuid",
    "bind+/var/empty:/proc/tty:ro,nodev,noexec,nosuid",
    "bind+/var/empty:/sys/dev/block:ro,nodev,noexec,nosuid",
    "bind+/var/empty:/sys/devices/virtual/powercap:ro,nodev,noexec,nosuid",
    "bind+/var/empty:/sys/firmware:ro,nodev,noexec,nosuid",
    "bind+/var/empty:/sys/fs/bpf:ro,nodev,noexec,nosuid",
    "bind+/var/empty:/sys/fs/ext4:ro,nodev,noexec,nosuid",
    "bind+/var/empty:/sys/fs/fuse:ro,nodev,noexec,nosuid",
    "bind+/var/empty:/sys/fs/pstore:ro,nodev,noexec,nosuid",
    "bind+/var/empty:/sys/fs/selinux:ro,nodev,noexec,nosuid",
    "bind+/var/empty:/sys/fs/smackfs:ro,nodev,noexec,nosuid",
    "deny/read,stat,write+/proc/1/**",
    "allow/read,stat+/proc/1/comm",
];

/// Landlock sandboxing profile, activated with:
/// `syd --profile landlock`.
pub const PROFILE_LANDLOCK: &[&str] = &[
    "sandbox/lock:on",
    "allow/lock/read+/bin",
    "allow/lock/read+/dev",
    "allow/lock/read+/etc",
    "allow/lock/read+/lib",
    "allow/lock/read+/opt",
    "allow/lock/read+/proc",
    "allow/lock/read+/run",
    "allow/lock/read+/sbin",
    "allow/lock/read+/sys",
    "allow/lock/read+/usr",
    "allow/lock/read+/var",
    "allow/lock/write+/dev/console",
    "allow/lock/write+/dev/fd",
    "allow/lock/write+/dev/full",
    "allow/lock/write+/dev/null",
    "allow/lock/write+/dev/ptmx",
    "allow/lock/write+/dev/pts",
    "allow/lock/write+/dev/shm",
    "allow/lock/write+/dev/tts",
    "allow/lock/write+/dev/tty",
    "allow/lock/write+/dev/zero",
    "allow/lock/write+/selinux/context",
    "allow/lock/write+/tmp",
];

/// Pand☮ra sandboxing profile.
pub const PROFILE_PANDORA: &[&str] = &["sandbox/read,stat,write,exec,net:on"];

/// Paludis sandboxing profile, activated with:
/// `syd --profile paludis`.
pub const PROFILE_PALUDIS: &[&str] = &[
    "lock:exec", // esandbox
    "sandbox/read,stat,write,net,tpe:on",
    "sandbox/exec:off", // turned on by esandbox in metadata phase.
    "trace/allow_safe_bind:true",
    "trace/allow_safe_kcapi:true",
    "trace/allow_unsafe_debug:true", // implies unsafe_{exec,memory}:1
    "trace/allow_unsupp_exec:true",
    "trace/allow_unsafe_env:true",
    "trace/allow_unsafe_filename:true", // sydbox#118
    "trace/allow_unsafe_kill:true",
    "trace/allow_unsafe_open:true",
    "trace/allow_unsafe_prlimit:true",
    "trace/allow_unsafe_socket:true",
    "trace/allow_unsupp_socket:true",
    // Allow loading dynamic libraries from sane locations on exec.
    r"allow/exec+/{lib*,opt,usr,var}/**/*.so{,\.[0-9]*}",
    "allow/read+/",
    "allow/read+/bin/***",
    "allow/read+/dev/***",
    "allow/read+/etc/***",
    "allow/read+/lib/***",
    "allow/read+/opt/***",
    "allow/read+/proc/***",
    "allow/read+/run/***",
    "allow/read+/sbin/***",
    "allow/read+/sys/***",
    "allow/read+/tmp/***",
    "allow/read+/usr/***",
    "allow/read+/var/***",
    "deny/read+/dev/*/***",
    "deny/read+/etc/crypttab",
    "deny/read+/etc/gshadow*",
    "deny/read+/etc/shadow*",
    "deny/read+/proc/*/***",
    "deny/read+/sys/*/***",
    "deny/read+/var/log/***",
    "allow/read+/dev/console",
    "allow/read+/dev/fd/**",
    "allow/read+/dev/full",
    "allow/read+/dev/null",
    "allow/read+/dev/kmsg",
    "allow/read+/dev/ptmx",
    "allow/read+/dev/pts/***",
    "allow/read+/dev/pty*",
    "allow/read+/dev/shm/***",
    "allow/read+/dev/stderr",
    "allow/read+/dev/stdout",
    "allow/read+/dev/tts",
    "allow/read+/dev/tty*",
    "allow/read+/dev/{,u}random",
    "allow/read+/dev/zero",
    "allow/read+/proc/[0-9]*",
    "deny/read+/proc/[0-9]*/*",
    "allow/read+/proc/[0-9]*/[ug]id_map",
    "allow/read+/proc/[0-9]*/attr/***",
    "allow/read+/proc/[0-9]*/cgroup",
    "allow/read+/proc/[0-9]*/clear_refs",
    "allow/read+/proc/[0-9]*/cmdline",
    "allow/read+/proc/[0-9]*/comm",
    "allow/read+/proc/[0-9]*/coredump_filter",
    "allow/read+/proc/[0-9]*/coredump_filter",
    "allow/read+/proc/[0-9]*/cpuset",
    "allow/read+/proc/[0-9]*/cwd",
    "allow/read+/proc/[0-9]*/exe",
    "allow/read+/proc/[0-9]*/fd/***",
    "allow/read+/proc/[0-9]*/fdinfo/***",
    "allow/read+/proc/[0-9]*/io",
    "allow/read+/proc/[0-9]*/limits",
    "allow/read+/proc/[0-9]*/loginuid",
    "allow/read+/proc/[0-9]*/map_files/***",
    "allow/read+/proc/[0-9]*/mounts/***",
    "allow/read+/proc/[0-9]*/mountinfo",
    "allow/read+/proc/[0-9]*/net/***",
    "allow/read+/proc/[0-9]*/ns/***",
    "allow/read+/proc/[0-9]*/oom_*",
    "allow/read+/proc/[0-9]*/root",
    "allow/read+/proc/[0-9]*/sched{,stat}",
    "allow/read+/proc/[0-9]*/seccomp",
    "allow/read+/proc/[0-9]*/sessionid",
    "allow/read+/proc/[0-9]*/setgroups",
    "allow/read+/proc/[0-9]*/stat{,m,us}",
    "allow/read+/proc/[0-9]*/task",
    "allow/read+/proc/[0-9]*/task/*",
    "allow/read+/proc/[0-9]*/time*",
    "allow/read+/proc/cgroups",
    "allow/read+/proc/cmdline",
    "allow/read+/proc/consoles",
    "allow/read+/proc/devices",
    "allow/read+/proc/diskstats",
    "allow/read+/proc/execdomains",
    "allow/read+/proc/fb",
    "allow/read+/proc/filesystems",
    "allow/read+/proc/interrupts",
    "allow/read+/proc/kmsg",
    "allow/read+/proc/loadavg",
    "allow/read+/proc/mounts/***",
    "allow/read+/proc/net/***",
    "allow/read+/proc/stat",
    "allow/read+/proc/sys",
    "allow/read+/proc/sys/{fs,kernel,vm}/***",
    "allow/read+/proc/uptime",
    "allow/read+/proc/version",
    "allow/read+/proc/{,thread-}self",
    "allow/read+/proc/{cpu,mem}info",
    "allow/read+/sys/bus",
    "allow/read+/sys/bus/{cpu,event_source}/***",
    "allow/read+/sys/class/i2c-adapter",
    "allow/read+/sys/class/power_supply/***",
    "allow/read+/sys/devices",
    "allow/read+/sys/devices/system",
    "allow/read+/sys/devices/system/cpu",
    "allow/read+/sys/devices/system/cpu/{isolated,kernel_max,online,offline,possible,present}",
    "allow/read+/sys/devices/system/node/***",
    "allow/read+/sys/fs",
    "allow/read+/sys/fs/cgroup/***",
    "allow/read+/sys/kernel/mm/transparent_hugepage/***",
    "allow/read+/sys/kernel/security/apparmor/***",
    "allow/stat+/",
    "allow/stat+/bin/***",
    "allow/stat+/dev/***",
    "allow/stat+/etc/***",
    "allow/stat+/lib/***",
    "allow/stat+/opt/***",
    "allow/stat+/proc/***",
    "allow/stat+/run/***",
    "allow/stat+/sbin/***",
    "allow/stat+/sys/***",
    "allow/stat+/tmp/***",
    "allow/stat+/usr/***",
    "allow/stat+/var/***",
    "deny/stat+/dev/*/***",
    "deny/stat+/etc/crypttab",
    "deny/stat+/etc/gshadow*",
    "deny/stat+/etc/shadow*",
    "deny/stat+/proc/*/***",
    "deny/stat+/sys/*/***",
    "deny/stat+/var/log/***",
    "allow/stat+/dev/console",
    "allow/stat+/dev/fd/**",
    "allow/stat+/dev/full",
    "allow/stat+/dev/null",
    "allow/stat+/dev/kmsg",
    "allow/stat+/dev/ptmx",
    "allow/stat+/dev/pts/***",
    "allow/stat+/dev/pty*",
    "allow/stat+/dev/shm/***",
    "allow/stat+/dev/stderr",
    "allow/stat+/dev/stdout",
    "allow/stat+/dev/tts",
    "allow/stat+/dev/tty*",
    "allow/stat+/dev/{,u}random",
    "allow/stat+/dev/zero",
    "allow/stat+/proc/[0-9]*",
    "deny/stat+/proc/[0-9]*/*",
    "allow/stat+/proc/[0-9]*/[ug]id_map",
    "allow/stat+/proc/[0-9]*/attr/***",
    "allow/stat+/proc/[0-9]*/cgroup",
    "allow/stat+/proc/[0-9]*/clear_refs",
    "allow/stat+/proc/[0-9]*/cmdline",
    "allow/stat+/proc/[0-9]*/comm",
    "allow/stat+/proc/[0-9]*/coredump_filter",
    "allow/stat+/proc/[0-9]*/coredump_filter",
    "allow/stat+/proc/[0-9]*/cpuset",
    "allow/stat+/proc/[0-9]*/cwd",
    "allow/stat+/proc/[0-9]*/exe",
    "allow/stat+/proc/[0-9]*/fd/***",
    "allow/stat+/proc/[0-9]*/fdinfo/***",
    "allow/stat+/proc/[0-9]*/io",
    "allow/stat+/proc/[0-9]*/limits",
    "allow/stat+/proc/[0-9]*/loginuid",
    "allow/stat+/proc/[0-9]*/map_files/***",
    "allow/stat+/proc/[0-9]*/mounts/***",
    "allow/stat+/proc/[0-9]*/mountinfo",
    "allow/stat+/proc/[0-9]*/net/***",
    "allow/stat+/proc/[0-9]*/ns/***",
    "allow/stat+/proc/[0-9]*/oom_*",
    "allow/stat+/proc/[0-9]*/root",
    "allow/stat+/proc/[0-9]*/sched{,stat}",
    "allow/stat+/proc/[0-9]*/seccomp",
    "allow/stat+/proc/[0-9]*/sessionid",
    "allow/stat+/proc/[0-9]*/setgroups",
    "allow/stat+/proc/[0-9]*/stat{,m,us}",
    "allow/stat+/proc/[0-9]*/task",
    "allow/stat+/proc/[0-9]*/task/*",
    "allow/stat+/proc/[0-9]*/time*",
    "allow/stat+/proc/cgroups",
    "allow/stat+/proc/cmdline",
    "allow/stat+/proc/consoles",
    "allow/stat+/proc/devices",
    "allow/stat+/proc/diskstats",
    "allow/stat+/proc/execdomains",
    "allow/stat+/proc/fb",
    "allow/stat+/proc/filesystems",
    "allow/stat+/proc/interrupts",
    "allow/stat+/proc/kmsg",
    "allow/stat+/proc/loadavg",
    "allow/stat+/proc/mounts/***",
    "allow/stat+/proc/net/***",
    "allow/stat+/proc/stat",
    "allow/stat+/proc/sys",
    "allow/stat+/proc/sys/{fs,kernel,vm}/***",
    "allow/stat+/proc/uptime",
    "allow/stat+/proc/version",
    "allow/stat+/proc/{,thread-}self",
    "allow/stat+/proc/{cpu,mem}info",
    "allow/stat+/sys/bus",
    "allow/stat+/sys/bus/{cpu,event_source}/***",
    "allow/stat+/sys/class/i2c-adapter",
    "allow/stat+/sys/class/power_supply/***",
    "allow/stat+/sys/devices",
    "allow/stat+/sys/devices/system",
    "allow/stat+/sys/devices/system/cpu",
    "allow/stat+/sys/devices/system/cpu/{isolated,kernel_max,online,offline,possible,present}",
    "allow/stat+/sys/devices/system/node/***",
    "allow/stat+/sys/fs",
    "allow/stat+/sys/fs/cgroup/***",
    "allow/stat+/sys/kernel/mm/transparent_hugepage/***",
    "allow/stat+/sys/kernel/security/apparmor/***",
    "allow/write+/dev/console",
    "allow/write+/dev/fd/**",
    "allow/write+/dev/full",
    "allow/write+/dev/null",
    "allow/write+/dev/ptmx",
    "allow/write+/dev/pts/**",
    "allow/write+/dev/pty*",
    "allow/write+/dev/shm/**",
    "allow/write+/dev/stderr",
    "allow/write+/dev/stdout",
    "allow/write+/dev/tts",
    "allow/write+/dev/tty*",
    "allow/write+/dev/zero",
    "allow/write+/proc/[0-9]*/[ug]id_map",
    "allow/write+/proc/[0-9]*/attr/***",
    "allow/write+/proc/[0-9]*/fd/***",
    "allow/write+/proc/[0-9]*/oom_score_adj",
    "allow/write+/proc/[0-9]*/setgroups",
    "allow/write+/proc/[0-9]*/task/***",
    "allow/write+/selinux/context/**",
    "allow/write+/tmp/***",
    "allow/write+/var/cache/**",
    "allow/write+/var/tmp/***",
    "allow/net/bind+loopback!0",
    "allow/net/bind+loopback!1024-65535",
    "allow/net/connect+/run/nscd/socket",
    "allow/net/connect+/var/lib/sss/pipes/nss",
    "allow/net/connect+/var/run/nscd/socket",
    // allow getaddrinfo() with AI_ADDRCONFIG on musl systems.
    "allow/net/connect+loopback!65535",
    // allow ping
    "allow/net/connect+any!0",
    "allow/net/connect+any!1025",
    // filter noisy systemd access
    "filter/net/connect+/run/systemd/userdb/io.systemd.DynamicUser",
];

/// User sandboxing profile, activated with:
/// `syd --profile user`.
pub const PROFILE_USER: &[&str] = &[
    "sandbox/read,stat,write,exec,net,lock,tpe:on",
    "trace/allow_safe_bind:true",
    "trace/allow_safe_kcapi:true",
    "allow/lock/read+/bin",
    "allow/lock/read+/sbin",
    "allow/lock/read+/dev",
    "allow/lock/read+/etc",
    "allow/lock/read+/home",
    "allow/lock/read+/lib",
    "allow/lock/read+/opt",
    "allow/lock/read+/run",
    "allow/lock/read+/snap",
    "allow/lock/read+/sys",
    "allow/lock/read+/usr",
    "allow/lock/read+/var",
    "allow/lock/write+/dev/console",
    "allow/lock/write+/dev/dri",
    "allow/lock/write+/dev/fd",
    "allow/lock/write+/dev/full",
    "allow/lock/write+/dev/null",
    "allow/lock/write+/dev/ptmx",
    "allow/lock/write+/dev/pts",
    "allow/lock/write+/dev/shm",
    "allow/lock/write+/dev/snd",
    "allow/lock/write+/dev/tts",
    "allow/lock/write+/dev/tty",
    "allow/lock/write+/dev/zero",
    "allow/lock/write+/proc",
    "allow/lock/write+/run/user",
    "allow/lock/write+/selinux/context",
    "allow/lock/write+/tmp",
    "allow/lock/write+/var/cache",
    "allow/lock/write+/var/tmp",
    "allow/read+/***",
    "deny/read+/dev/*/***",
    "deny/read+/etc/crypttab",
    "deny/read+/etc/gshadow*",
    "deny/read+/etc/shadow*",
    "deny/read+/proc/*/***",
    "deny/read+/sys/*/***",
    "deny/read+/var/log/***",
    "allow/read+/dev/console",
    "allow/read+/dev/dri/***",
    "allow/read+/dev/fd/**",
    "allow/read+/dev/full",
    "allow/read+/dev/input/***",
    "allow/read+/dev/null",
    "allow/read+/dev/kmsg",
    "allow/read+/dev/ptmx",
    "allow/read+/dev/pts/***",
    "allow/read+/dev/pty*",
    "allow/read+/dev/shm/***",
    "allow/read+/dev/snd/***",
    "allow/read+/dev/stderr",
    "allow/read+/dev/stdout",
    "allow/read+/dev/tts",
    "allow/read+/dev/tty*",
    "allow/read+/dev/zero",
    "allow/read+/dev/{,u}random",
    "allow/read+/proc/[0-9]*",
    "deny/read+/proc/[0-9]*/*",
    "allow/read+/proc/[0-9]*/[ug]id_map",
    "allow/read+/proc/[0-9]*/attr/***",
    "allow/read+/proc/[0-9]*/cgroup",
    "allow/read+/proc/[0-9]*/clear_refs",
    "allow/read+/proc/[0-9]*/cmdline",
    "allow/read+/proc/[0-9]*/comm",
    "allow/read+/proc/[0-9]*/coredump_filter",
    "allow/read+/proc/[0-9]*/coredump_filter",
    "allow/read+/proc/[0-9]*/cpuset",
    "allow/read+/proc/[0-9]*/cwd",
    "allow/read+/proc/[0-9]*/exe",
    "allow/read+/proc/[0-9]*/fd/***",
    "allow/read+/proc/[0-9]*/fdinfo/***",
    "allow/read+/proc/[0-9]*/io",
    "allow/read+/proc/[0-9]*/limits",
    "allow/read+/proc/[0-9]*/loginuid",
    "allow/read+/proc/[0-9]*/map_files/***",
    "allow/read+/proc/[0-9]*/mounts/***",
    "allow/read+/proc/[0-9]*/mountinfo",
    "allow/read+/proc/[0-9]*/net/***",
    "allow/read+/proc/[0-9]*/ns/***",
    "allow/read+/proc/[0-9]*/oom_*",
    "allow/read+/proc/[0-9]*/root",
    "allow/read+/proc/[0-9]*/sched{,stat}",
    "allow/read+/proc/[0-9]*/seccomp",
    "allow/read+/proc/[0-9]*/sessionid",
    "allow/read+/proc/[0-9]*/setgroups",
    "allow/read+/proc/[0-9]*/stat{,m,us}",
    "allow/read+/proc/[0-9]*/task",
    "allow/read+/proc/[0-9]*/task/*",
    "allow/read+/proc/[0-9]*/time*",
    "allow/read+/proc/cgroups",
    "allow/read+/proc/cmdline",
    "allow/read+/proc/consoles",
    "allow/read+/proc/devices",
    "allow/read+/proc/diskstats",
    "allow/read+/proc/execdomains",
    "allow/read+/proc/fb",
    "allow/read+/proc/filesystems",
    "allow/read+/proc/interrupts",
    "allow/read+/proc/kmsg",
    "allow/read+/proc/loadavg",
    "allow/read+/proc/mounts/***",
    "allow/read+/proc/net/***",
    "allow/read+/proc/stat",
    "allow/read+/proc/sys",
    "allow/read+/proc/sys/{fs,kernel,vm}/***",
    "allow/read+/proc/uptime",
    "allow/read+/proc/version",
    "allow/read+/proc/{,thread-}self",
    "allow/read+/proc/{cpu,mem}info",
    "allow/read+/sys/{bus,class,dev,devices,fs}/***",
    "allow/read+/sys/class/***",
    "allow/read+/sys/dev/***",
    "allow/read+/sys/devices/***",
    "allow/read+/sys/fs/***",
    "allow/read+/sys/kernel/mm/transparent_hugepage/***",
    "allow/read+/sys/kernel/security/apparmor/***",
    "allow/stat+/***",
    "deny/stat+/dev/*/***",
    "deny/stat+/proc/*/***",
    "deny/stat+/sys/*/***",
    "deny/stat+/var/log/***",
    "allow/stat+/dev/console",
    "allow/stat+/dev/dri/***",
    "allow/stat+/dev/fd/**",
    "allow/stat+/dev/full",
    "allow/stat+/dev/input/***",
    "allow/stat+/dev/null",
    "allow/stat+/dev/kmsg",
    "allow/stat+/dev/ptmx",
    "allow/stat+/dev/pts/***",
    "allow/stat+/dev/pty*",
    "allow/stat+/dev/shm/***",
    "allow/stat+/dev/snd/***",
    "allow/stat+/dev/stderr",
    "allow/stat+/dev/stdout",
    "allow/stat+/dev/tts",
    "allow/stat+/dev/tty*",
    "allow/stat+/dev/zero",
    "allow/stat+/dev/{,u}random",
    "allow/stat+/proc/[0-9]*",
    "deny/stat+/proc/[0-9]*/*",
    "allow/stat+/proc/[0-9]*/[ug]id_map",
    "allow/stat+/proc/[0-9]*/attr/***",
    "allow/stat+/proc/[0-9]*/cgroup",
    "allow/stat+/proc/[0-9]*/clear_refs",
    "allow/stat+/proc/[0-9]*/cmdline",
    "allow/stat+/proc/[0-9]*/comm",
    "allow/stat+/proc/[0-9]*/coredump_filter",
    "allow/stat+/proc/[0-9]*/coredump_filter",
    "allow/stat+/proc/[0-9]*/cpuset",
    "allow/stat+/proc/[0-9]*/cwd",
    "allow/stat+/proc/[0-9]*/exe",
    "allow/stat+/proc/[0-9]*/fd/***",
    "allow/stat+/proc/[0-9]*/fdinfo/***",
    "allow/stat+/proc/[0-9]*/io",
    "allow/stat+/proc/[0-9]*/limits",
    "allow/stat+/proc/[0-9]*/loginuid",
    "allow/stat+/proc/[0-9]*/map_files/***",
    "allow/stat+/proc/[0-9]*/mounts/***",
    "allow/stat+/proc/[0-9]*/mountinfo",
    "allow/stat+/proc/[0-9]*/net/***",
    "allow/stat+/proc/[0-9]*/ns/***",
    "allow/stat+/proc/[0-9]*/oom_*",
    "allow/stat+/proc/[0-9]*/root",
    "allow/stat+/proc/[0-9]*/sched{,stat}",
    "allow/stat+/proc/[0-9]*/seccomp",
    "allow/stat+/proc/[0-9]*/sessionid",
    "allow/stat+/proc/[0-9]*/setgroups",
    "allow/stat+/proc/[0-9]*/stat{,m,us}",
    "allow/stat+/proc/[0-9]*/task",
    "allow/stat+/proc/[0-9]*/task/*",
    "allow/stat+/proc/[0-9]*/time*",
    "allow/stat+/proc/cgroups",
    "allow/stat+/proc/cmdline",
    "allow/stat+/proc/consoles",
    "allow/stat+/proc/devices",
    "allow/stat+/proc/diskstats",
    "allow/stat+/proc/execdomains",
    "allow/stat+/proc/fb",
    "allow/stat+/proc/filesystems",
    "allow/stat+/proc/interrupts",
    "allow/stat+/proc/kmsg",
    "allow/stat+/proc/loadavg",
    "allow/stat+/proc/mounts/***",
    "allow/stat+/proc/net/***",
    "allow/stat+/proc/stat",
    "allow/read+/proc/sys",
    "allow/read+/proc/sys/{fs,kernel,vm}/***",
    "allow/stat+/proc/uptime",
    "allow/stat+/proc/version",
    "allow/stat+/proc/{,thread-}self",
    "allow/stat+/proc/{cpu,mem}info",
    "allow/stat+/sys/{bus,class,dev,devices,fs}/***",
    "allow/stat+/sys/kernel/mm/transparent_hugepage/***",
    "allow/stat+/sys/kernel/security/apparmor/***",
    "allow/exec+/bin/**",
    "allow/exec+/lib*/**",
    "allow/exec+/sbin/**",
    "allow/exec+/snap/**",
    "allow/exec+/usr/**/bin/**",
    "allow/exec+/usr/**/lib*/**",
    "allow/exec+/usr/**/sbin/**",
    "allow/exec+/usr/games/**",
    "allow/write+/dev/console",
    "allow/write+/dev/dri/**",
    "allow/write+/dev/fd/**",
    "allow/write+/dev/full",
    "allow/write+/dev/hidraw[0-9]*",
    "allow/write+/dev/null",
    "allow/write+/dev/ptmx",
    "allow/write+/dev/pts/**",
    "allow/write+/dev/pty*",
    "allow/write+/dev/shm/**",
    "allow/write+/dev/snd/**",
    "allow/write+/dev/stderr",
    "allow/write+/dev/stdout",
    "allow/write+/dev/tts",
    "allow/write+/dev/tty*",
    "allow/write+/dev/zero",
    "allow/write+/proc/[0-9]*/[ug]id_map",
    "allow/write+/proc/[0-9]*/attr/***",
    "allow/write+/proc/[0-9]*/fd/***",
    "allow/write+/proc/[0-9]*/oom_score_adj",
    "allow/write+/proc/[0-9]*/setgroups",
    "allow/write+/proc/[0-9]*/task/***",
    "allow/write+/run/**",
    "allow/write+/selinux/context/**",
    "allow/write+/tmp/***",
    "allow/write+/var/cache/**",
    "allow/write+/var/tmp/***",
    "allow/net/bind+loopback!0",
    "allow/net/bind+loopback!1024-65535",
    "allow/net/connect+/run/nscd/socket",
    "allow/net/connect+/var/lib/sss/pipes/nss",
    "allow/net/connect+/var/run/nscd/socket",
    // allow getaddrinfo() with AI_ADDRCONFIG on musl systems.
    "allow/net/connect+loopback!65535",
    // allow ping
    "allow/net/connect+any!0",
    "allow/net/connect+any!1025",
    // Allow UNIX socket access to run & /tmp (think X, tmux, screen et a.)
    "allow/net/bind+/run/**",
    "allow/net/bind+/tmp/**",
    "allow/net/connect+/run/**",
    "allow/net/connect+/tmp/**",
    // Allow access to syslog
    "allow/net/connect+/dev/log",
    // Allow access to systemd journal
    "allow/read+/var/log/journal/***",
    // Allow access to PulseAudio
    "allow/net/connect+/var/run/pulse/native",
];

/// The list of perf system calls.
pub const PERF_SYSCALLS: &[&str] = &["perf_event_open"];

/// The list of ptrace system calls.
pub const PTRACE_SYSCALLS: &[&str] = &["ptrace", "process_vm_readv", "process_vm_writev"];

/// The list of system calls which are of the getid family.
pub const GET_ID_SYSCALLS: &[&str] = &[
    "getuid",
    "getuid32",
    "getgid",
    "getgid32",
    "geteuid",
    "geteuid32",
    "getegid",
    "getegid32",
];

/// OCI default sandboxing profile
/// Use `syd -p oci`
pub const PROFILE_OCI: &[&str] = &[
    "allow/exec+/bin/**",
    "allow/exec+/lib*/**",
    "allow/exec+/sbin/**",
    "allow/exec+/snap/**",
    "allow/exec+/usr/**/bin/**",
    "allow/exec+/usr/**/lib*/**",
    "allow/exec+/usr/**/sbin/**",
    "allow/exec+/usr/games/**",
    // Allow loading dynamic libraries from sane locations on exec.
    r"allow/exec+/{lib*,opt,usr,var}/**/*.so{,\.[0-9]*}",
    "allow/read+/",
    "allow/read+/bin/***",
    "allow/read+/dev/***",
    "allow/read+/etc/***",
    "allow/read+/lib/***",
    "allow/read+/opt/***",
    "allow/read+/proc/***",
    "allow/read+/run/***",
    "allow/read+/sbin/***",
    "allow/read+/sys/***",
    "allow/read+/tmp/***",
    "allow/read+/usr/***",
    "allow/read+/var/***",
    "deny/read+/dev/*/***",
    "deny/read+/etc/crypttab",
    "deny/read+/etc/gshadow*",
    "deny/read+/etc/shadow*",
    "deny/read+/proc/*/***",
    "deny/read+/sys/*/***",
    "deny/read+/var/log/***",
    "allow/read+/dev/console",
    "allow/read+/dev/fd/**",
    "allow/read+/dev/full",
    "allow/read+/dev/null",
    "allow/read+/dev/ptmx",
    "allow/read+/dev/pts/***",
    "allow/read+/dev/pty*",
    "allow/read+/dev/shm/***",
    "allow/read+/dev/stderr",
    "allow/read+/dev/stdout",
    "allow/read+/dev/tts",
    "allow/read+/dev/tty*",
    "allow/read+/dev/urandom",
    "allow/read+/dev/zero",
    "allow/read+/proc/[0-9]*",
    "deny/read+/proc/[0-9]*/*",
    "allow/read+/proc/[0-9]*/attr/***",
    "allow/read+/proc/[0-9]*/cgroup",
    "allow/read+/proc/[0-9]*/clear_refs",
    "allow/read+/proc/[0-9]*/cmdline",
    "allow/read+/proc/[0-9]*/comm",
    "allow/read+/proc/[0-9]*/coredump_filter",
    "allow/read+/proc/[0-9]*/coredump_filter",
    "allow/read+/proc/[0-9]*/cpuset",
    "allow/read+/proc/[0-9]*/cwd",
    "allow/read+/proc/[0-9]*/exe",
    "allow/read+/proc/[0-9]*/fd/***",
    "allow/read+/proc/[0-9]*/fdinfo/***",
    "allow/read+/proc/[0-9]*/io",
    "allow/read+/proc/[0-9]*/limits",
    "allow/read+/proc/[0-9]*/loginuid",
    "allow/read+/proc/[0-9]*/map_files/***",
    "allow/read+/proc/[0-9]*/mounts/***",
    "allow/read+/proc/[0-9]*/mountinfo",
    "allow/read+/proc/[0-9]*/net/***",
    "allow/read+/proc/[0-9]*/ns/***",
    "allow/read+/proc/[0-9]*/oom_*",
    "allow/read+/proc/[0-9]*/root",
    "allow/read+/proc/[0-9]*/sched{,stat}",
    "allow/read+/proc/[0-9]*/seccomp",
    "allow/read+/proc/[0-9]*/sessionid",
    "allow/read+/proc/[0-9]*/stat{,m,us}",
    "allow/read+/proc/[0-9]*/task",
    "allow/read+/proc/[0-9]*/task/*",
    "allow/read+/proc/[0-9]*/time*",
    "allow/read+/proc/cgroups",
    "allow/read+/proc/cmdline",
    "allow/read+/proc/consoles",
    "allow/read+/proc/devices",
    "allow/read+/proc/diskstats",
    "allow/read+/proc/execdomains",
    "allow/read+/proc/fb",
    "allow/read+/proc/filesystems",
    "allow/read+/proc/interrupts",
    "allow/read+/proc/loadavg",
    "allow/read+/proc/mounts/***",
    "allow/read+/proc/net/***",
    "allow/read+/proc/stat",
    "allow/read+/proc/sys",
    "allow/read+/proc/sys/{fs,kernel,vm}/***",
    "allow/read+/proc/uptime",
    "allow/read+/proc/version",
    "allow/read+/proc/{,thread-}self",
    "allow/read+/proc/{cpu,mem}info",
    "allow/read+/sys/bus",
    "allow/read+/sys/bus/{cpu,event_source}/***",
    "allow/read+/sys/class/i2c-adapter",
    "allow/read+/sys/class/power_supply/***",
    "allow/read+/sys/devices",
    "allow/read+/sys/devices/system",
    "allow/read+/sys/devices/system/cpu",
    "allow/read+/sys/devices/system/cpu/{isolated,kernel_max,online,offline,possible,present}",
    "allow/read+/sys/devices/system/node/***",
    "allow/read+/sys/fs",
    "allow/read+/sys/fs/cgroup/***",
    "allow/read+/sys/kernel/mm/transparent_hugepage/***",
    "allow/read+/sys/kernel/security/apparmor/***",
    "allow/stat+/",
    "allow/stat+/bin/***",
    "allow/stat+/dev/***",
    "allow/stat+/etc/***",
    "allow/stat+/lib/***",
    "allow/stat+/opt/***",
    "allow/stat+/proc/***",
    "allow/stat+/run/***",
    "allow/stat+/sbin/***",
    "allow/stat+/sys/***",
    "allow/stat+/tmp/***",
    "allow/stat+/usr/***",
    "allow/stat+/var/***",
    "deny/stat+/dev/*/***",
    "deny/stat+/etc/crypttab",
    "deny/stat+/etc/gshadow*",
    "deny/stat+/etc/shadow*",
    "deny/stat+/proc/*/***",
    "deny/stat+/sys/*/***",
    "deny/stat+/var/log/***",
    "allow/stat+/dev/console",
    "allow/stat+/dev/fd/**",
    "allow/stat+/dev/full",
    "allow/stat+/dev/null",
    "allow/stat+/dev/ptmx",
    "allow/stat+/dev/pts/***",
    "allow/stat+/dev/pty*",
    "allow/stat+/dev/shm/***",
    "allow/stat+/dev/stderr",
    "allow/stat+/dev/stdout",
    "allow/stat+/dev/tts",
    "allow/stat+/dev/tty*",
    "allow/stat+/dev/{,u}random",
    "allow/stat+/dev/zero",
    "allow/stat+/proc/[0-9]*",
    "deny/stat+/proc/[0-9]*/*",
    "allow/stat+/proc/[0-9]*/attr/***",
    "allow/stat+/proc/[0-9]*/cgroup",
    "allow/stat+/proc/[0-9]*/clear_refs",
    "allow/stat+/proc/[0-9]*/cmdline",
    "allow/stat+/proc/[0-9]*/comm",
    "allow/stat+/proc/[0-9]*/coredump_filter",
    "allow/stat+/proc/[0-9]*/coredump_filter",
    "allow/stat+/proc/[0-9]*/cpuset",
    "allow/stat+/proc/[0-9]*/cwd",
    "allow/stat+/proc/[0-9]*/exe",
    "allow/stat+/proc/[0-9]*/fd/***",
    "allow/stat+/proc/[0-9]*/fdinfo/***",
    "allow/stat+/proc/[0-9]*/io",
    "allow/stat+/proc/[0-9]*/limits",
    "allow/stat+/proc/[0-9]*/loginuid",
    "allow/stat+/proc/[0-9]*/map_files/***",
    "allow/stat+/proc/[0-9]*/mounts/***",
    "allow/stat+/proc/[0-9]*/mountinfo",
    "allow/stat+/proc/[0-9]*/net/***",
    "allow/stat+/proc/[0-9]*/ns/***",
    "allow/stat+/proc/[0-9]*/oom_*",
    "allow/stat+/proc/[0-9]*/root",
    "allow/stat+/proc/[0-9]*/sched{,stat}",
    "allow/stat+/proc/[0-9]*/seccomp",
    "allow/stat+/proc/[0-9]*/sessionid",
    "allow/stat+/proc/[0-9]*/stat{,m,us}",
    "allow/stat+/proc/[0-9]*/task",
    "allow/stat+/proc/[0-9]*/task/*",
    "allow/stat+/proc/[0-9]*/time*",
    "allow/stat+/proc/cgroups",
    "allow/stat+/proc/cmdline",
    "allow/stat+/proc/consoles",
    "allow/stat+/proc/devices",
    "allow/stat+/proc/diskstats",
    "allow/stat+/proc/execdomains",
    "allow/stat+/proc/fb",
    "allow/stat+/proc/filesystems",
    "allow/stat+/proc/interrupts",
    "allow/stat+/proc/loadavg",
    "allow/stat+/proc/mounts/***",
    "allow/stat+/proc/net/***",
    "allow/stat+/proc/stat",
    "allow/stat+/proc/sys",
    "allow/stat+/proc/sys/{fs,kernel,vm}/***",
    "allow/stat+/proc/uptime",
    "allow/stat+/proc/version",
    "allow/stat+/proc/{,thread-}self",
    "allow/stat+/proc/{cpu,mem}info",
    "allow/stat+/sys/bus",
    "allow/stat+/sys/bus/{cpu,event_source}/***",
    "allow/stat+/sys/class/i2c-adapter",
    "allow/stat+/sys/class/power_supply/***",
    "allow/stat+/sys/devices",
    "allow/stat+/sys/devices/system",
    "allow/stat+/sys/devices/system/cpu",
    "allow/stat+/sys/devices/system/cpu/{isolated,kernel_max,online,offline,possible,present}",
    "allow/stat+/sys/devices/system/node/***",
    "allow/stat+/sys/fs",
    "allow/stat+/sys/fs/cgroup/***",
    "allow/stat+/sys/kernel/mm/transparent_hugepage/***",
    "allow/stat+/sys/kernel/security/apparmor/***",
    "allow/write+/dev/console",
    "allow/write+/dev/fd/**",
    "allow/write+/dev/full",
    "allow/write+/dev/null",
    "allow/write+/dev/ptmx",
    "allow/write+/dev/pts/**",
    "allow/write+/dev/pty*",
    "allow/write+/dev/shm/**",
    "allow/write+/dev/stderr",
    "allow/write+/dev/stdout",
    "allow/write+/dev/tts",
    "allow/write+/dev/tty*",
    "allow/write+/dev/zero",
    "allow/write+/proc/[0-9]*/attr/***",
    "allow/write+/proc/[0-9]*/fd/***",
    "allow/write+/proc/[0-9]*/oom_score_adj",
    "allow/write+/proc/[0-9]*/task/***",
    "allow/write+/selinux/context/**",
    "allow/write+/tmp/***",
    "allow/write+/var/cache/**",
    "allow/write+/var/tmp/***",
    "allow/net/bind+loopback!0",
    "allow/net/bind+loopback!1024-65535",
    "allow/net/connect+/run/nscd/socket",
    "allow/net/connect+/var/lib/sss/pipes/nss",
    "allow/net/connect+/var/run/nscd/socket",
    // allow getaddrinfo() with AI_ADDRCONFIG on musl systems.
    "allow/net/connect+loopback!65535",
    // allow ping
    "allow/net/connect+any!0",
    "allow/net/connect+any!1025",
    // filter noisy systemd access
    "filter/net/connect+/run/systemd/userdb/io.systemd.DynamicUser",
    // hide and mask pid=1 aka syd-oci.
    "deny/read,stat,write+/proc/1/***",
    "mask+/proc/1/***",
    // hide and mask /.dockerenv
    "deny/read,stat,write+/.dockerenv",
    "mask+/.dockerenv",
];

/// The list of system calls which are of the setid family.
pub const SET_ID_SYSCALLS: &[&str] = &[
    "setfsgid",
    "setfsgid32",
    "setfsuid",
    "setfsuid32",
    "setgid",
    "setgid32",
    "setregid",
    "setregid32",
    "setresgid",
    "setresgid32",
    "setresuid",
    "setresuid32",
    "setreuid",
    "setreuid32",
    "setuid",
    "setuid32",
    "setgroups",
    "setgroups32",
];

/// SAFETY: We do not support diverging FsID from Effective ID.
/// SAFETY: We do not support setgroups (due to pointer deref -> TOCTOU vector)
pub const UNSAFE_ID_SYSCALLS: &[&str] = &[
    "setfsgid",
    "setfsgid32",
    "setfsuid",
    "setfsuid32",
    "setgroups",
    "setgroups32",
];

/// The list of system calls which syd has a `UNotify` callback for
pub const HOOK_SYSCALLS: &[&str] = &[
    "bind",
    "connect",
    "sendto",
    "sendmsg",
    "sendmmsg",
    "execve",
    "execveat",
    "chdir",
    "fchdir",
    "getdents",
    "getdents64",
    "stat",
    "stat64",
    "fstat",
    "lstat",
    "statx",
    "newfstatat",
    "access",
    "faccessat",
    "faccessat2",
    "chmod",
    "fchmodat",
    "fchmodat2",
    "chown",
    "lchown",
    "fchownat",
    "creat",
    "link",
    "symlink",
    "unlink",
    "linkat",
    "symlinkat",
    "unlinkat",
    "mkdir",
    "rmdir",
    "mkdirat",
    "mknod",
    "mknodat",
    "open",
    "openat",
    "openat2",
    "rename",
    "renameat",
    "renameat2",
    "utime",
    "utimes",
    "futimesat",
    "utimensat",
    "truncate",
    "truncate64",
    "getxattr",
    "fgetxattr",
    "lgetxattr",
    "setxattr",
    "fsetxattr",
    "lsetxattr",
    "listxattr",
    "flistxattr",
    "llistxattr",
    "removexattr",
    "fremovexattr",
    "lremovexattr",
    "getrandom",
    "ioctl",
    "prctl",
    "kill",
    "tkill",
    "tgkill",
    "pidfd_open",
    "brk",
    "mmap",
    "mmap2",
    "mremap",
    "sync",
    "syncfs",
    "setrlimit",
    "prlimit64",
    "readlink",
    "readlinkat",
    "statfs",
    "statfs64",
    "fstatfs",
    "fstatfs64",
    "uname",
    "write",
    "writev",
    "fanotify_mark",
    "inotify_add_watch",
];

/// The list of system calls which are no-ops.
/// SAFETY: sync, syncfs are no-ops to prevent potential local DoS.
/// SAFETY: chroot, pivot_root are no-ops for compat with old daemons,
/// that change root into empty directory once all path access is done.
pub const NOOP_SYSCALLS: &[&str] = &["chroot", "pivot_root", "sync", "syncfs"];

/// The list of system calls that are denied in syd parent seccomp filter.
/// This filter is inherited by the sandbox process.
/// These system calls must not be used by syd.
pub const DEAD_SYSCALLS: &[&str] = &[
    "_sysctl",
    "acct",
    "add_key",
    "afs_syscall",
    "bpf",
    "create_module",
    "delete_module",
    "finit_module",
    "fsconfig",
    "fsmount",
    "fsopen",
    "fspick",
    "get_kernel_syms",
    "init_module",
    "ioperm",
    "iopl",
    "kexec_file_load",
    "kexec_load",
    "keyctl",
    "mbind",
    "migrate_pages",
    "mount_setattr",
    "move_pages",
    "name_to_handle_at",
    "nfsservctl",
    "open_tree",
    "putpmsg",
    "query_module",
    "quotactl",
    "quotactl_fd",
    "reboot",
    "request_key",
    "set_mempolicy",
    "swapoff",
    "swapon",
    "tuxcall",
    "umount",
    "umount2",
    "uselib",
    "userfaultfd",
    "vhangup",
    "vserver",
];

/// The list of system calls which are part of time/clock adjustment.
pub const TIME_SYSCALLS: &[&str] = &["adjtimex", "clock_adjtime", "clock_adjtime64"];

/// The list of system calls which are part of the io_uring interface.
pub const IOURING_SYSCALLS: &[&str] = &["io_uring_enter", "io_uring_register", "io_uring_setup"];

/// The list of system calls which are allowlisted without any filtering.
pub const SAFE_SYSCALLS: &[&str] = &[
    "_llseek",
    "_newselect",
    //"_sysctl",
    "accept",
    "accept4",
    //"acct",
    //add_key
    //adjtimex, restrictions applied, see setup_seccomp.
    //afs_syscall
    "alarm",
    "arch_prctl",
    "arm_fadvise64_64",
    "arm_sync_file_range",
    //"bpf",
    "breakpoint", // arm
    //"brk",
    "cacheflush", // arm
    "capget",
    "capset",
    //"chdir",
    //"chroot",
    //clock_adjtime, restrictions applied, see setup_seccomp.
    //clock_adjtime64, restrictions applied, see setup_seccomp.
    "clock_getres",
    "clock_getres_time64",
    "clock_gettime",
    "clock_gettime64",
    "clock_nanosleep",
    "clock_nanosleep_time64",
    "clock_settime",
    "clock_settime64",
    // "clone", subject to fork protection
    // "clone3", subject to fork protection
    "close",
    "close_range",
    "copy_file_range",
    //create_module
    //delete_module
    "dup",
    "dup2",
    "dup3",
    "epoll_create",
    "epoll_create1",
    "epoll_ctl",
    "epoll_ctl_old",
    "epoll_pwait",
    "epoll_pwait2",
    "epoll_wait",
    "epoll_wait_old",
    "eventfd",
    "eventfd2",
    "exit",
    "exit_group",
    "fadvise64",
    "fallocate",
    "fanotify_init",
    //"fanotify_mark", stat sandboxing
    "fchmod",
    "fchown",
    "fchown32",
    "fcntl",
    "fcntl64",
    "fdatasync",
    //finit_module
    "flock",
    //"fork", subject to fork protection
    //fsconfig
    //fsmount
    //fsopen
    //fspick
    //"fstatfs",
    //"fstatfs64",
    "fsync",
    "ftruncate",
    "ftruncate64",
    "futex",
    "futex_time64",
    //get_kernel_syms
    "get_mempolicy",
    "get_robust_list",
    "get_thread_area",
    "getcpu",
    "getcwd",
    //"getdents",
    //"getdents64",
    //"getegid",
    //"getegid32",
    //"geteuid",
    //"geteuid32",
    //"getgid",
    //"getgid32",
    "getgroups",
    "getgroups32",
    "getitimer",
    "getpeername",
    "getpgid",
    "getpgrp",
    "getpid",
    "getpmsg",
    "getppid",
    "getpriority",
    //"getrandom", restrictions applied, see setup_seccomp.
    "getresgid",
    "getresuid",
    "getrlimit",
    "getrusage",
    "getsid",
    "getuid",
    "getsockname",
    "getsockopt",
    "gettid",
    "gettimeofday",
    //init_module
    //"inotify_add_watch",
    "inotify_init",
    "inotify_init1",
    "inotify_rm_watch",
    "io_cancel",
    "io_destroy",
    "io_getevents",
    "io_pgetevents",
    "io_pgetevents_time64",
    "io_setup",
    "io_submit",
    /*
     * io-uring: restrictions applied, see setup_seccomp
    "io_uring_enter",
    "io_uring_register",
    "io_uring_setup",
    */
    //"ioctl", restrictions applied, see setup_seccomp.
    //"ioperm",
    //"iopl",
    "ioprio_get",
    "ioprio_set",
    "kcmp",
    //kexec_file_load,
    //kexec_load,
    //keyctl,
    //"kill", restrictions applied, see setup_seccomp
    "landlock_add_rule",
    "landlock_create_ruleset",
    "landlock_restrict_self",
    "listen",
    "lookup_dcookie",
    "lseek",
    "madvise",
    //"mbind",
    "membarrier",
    "memfd_create",
    "memfd_secret",
    //"migrate_pages",
    "mincore",
    "mlock",
    "mlock2",
    "mlockall",
    //"mmap",
    //"mmap2",
    "modify_ldt",
    //mount
    //mount_setattr
    //"move_pages",
    "mprotect",
    "mq_getsetattr",
    "mq_notify",
    "mq_open",
    "mq_timedreceive",
    "mq_timedreceive_time64",
    "mq_timedsend",
    "mq_timedsend_time64",
    "mq_unlink",
    //"mremap",
    "msgctl",
    "msgget",
    "msgrcv",
    "msgsnd",
    "msync",
    "munlock",
    "munlockall",
    "munmap",
    //TODO:name_to_handle_at
    "nanosleep",
    //nfsservctl,
    "open_by_handle_at",
    //open_tree
    "pause",
    //"perf_event_open", restrictions applied see load_seccomp_parent and setup_seccomp.
    "personality", // restrictions applied in see load_seccomp_parent.
    "pidfd_getfd",
    //"pidfd_open", restrictions applied, see setup_seccomp
    "pidfd_send_signal",
    "pipe",
    "pipe2",
    //pivot_root,
    "pkey_alloc",
    "pkey_free",
    "pkey_mprotect",
    "poll",
    "ppoll",
    "ppoll_time64",
    // "prctl", restrictions applied, see setup_seccomp.
    "pread64",
    "preadv",
    "preadv2",
    //"prlimit64",
    "process_madvise",
    "process_mrelease",
    //process_vm_readv // restrictions applied, see setup_seccomp.
    //process_vm_writev // ditto.
    "pselect6",
    "pselect6_time64",
    //"ptrace", // restrictions applied see load_seccomp_parent and setup_seccomp.
    //"putpmsg",
    "pwrite64",
    "pwritev",
    "pwritev2",
    //query_module,
    //quotactl
    //quotactl_fd
    "read",
    "readahead",
    //"readlink",
    //"readlinkat",
    "readv",
    //reboot
    "recv",
    "recvfrom",
    "recvmmsg",
    "recvmmsg_time64",
    "recvmsg",
    "remap_file_pages",
    //request_key
    "restart_syscall",
    "rseq",
    "rt_sigaction",
    "rt_sigpending",
    "rt_sigprocmask",
    "rt_sigqueueinfo",
    "rt_sigreturn",
    "rt_sigsuspend",
    "rt_sigtimedwait",
    "rt_sigtimedwait_time64",
    "rt_tgsigqueueinfo",
    "s390_pci_mmio_read",
    "s390_pci_mmio_write",
    "s390_runtime_instr",
    "sched_get_priority_max",
    "sched_get_priority_min",
    "sched_getaffinity",
    "sched_getattr",
    "sched_getparam",
    "sched_getscheduler",
    "sched_setaffinity",
    "sched_setattr",
    "sched_setscheduler",
    "sched_yield",
    "seccomp",
    //security
    "select",
    "semctl",
    "semget",
    "semop",
    "semtimedop",
    "semtimedop_time64",
    "send",
    "sendfile",
    "sendfile64",
    //"sendmmsg", emulated.
    //"sendmsg", emulated.
    //"set_mempolicy",
    "set_robust_list",
    "set_thread_area",
    "set_tid_address",
    "setdomainname",
    //"setfsgid",
    //"setfsgid32",
    //"setfsuid",
    //"setfsuid32",
    //"setgid",
    //"setgid32",
    //"setgroups",
    //"setgroups32",
    "sethostname",
    "setitimer",
    "setns",
    "setpgid",
    "setpriority",
    //"setregid",
    //"setregid32",
    //"setresgid",
    //"setresgid32",
    //"setresuid",
    //"setresuid32",
    //"setreuid",
    //"setreuid32",
    //"setrlimit",
    "setsid",
    "setsockopt",
    //"settimeofday"
    //"setuid",
    //"setuid32",
    "set_tls",  // arm
    "sgetmask", // x86
    "ssetmask", // x86
    "shmat",
    "shmctl",
    "shmdt",
    "shmget",
    "shutdown",
    "signal",
    "sigaction",
    "sigaltstack",
    "signalfd",
    "signalfd4",
    "sigpending",
    "sigprocmask",
    "sigsuspend",
    "sigreturn",
    "socket",
    "socketpair",
    "splice",
    //"statfs",
    //"statfs64",
    //swapoff
    //swapon
    //"sync",
    "sync_file_range",
    //"syncfs",
    "sysinfo",
    //"syslog",
    "tee",
    //"tgkill", restrictions applied, see setup_seccomp
    "time",
    "timer_create",
    "timer_delete",
    "timer_getoverrun",
    "timer_gettime",
    "timer_gettime64",
    "timer_settime",
    "timer_settime64",
    "timerfd_create",
    "timerfd_gettime",
    "timerfd_gettime64",
    "timerfd_settime",
    "timerfd_settime64",
    "times",
    //"tkill", an obsolete predecessor to tgkill, should be avoided.
    //tuxcall
    "ugetrlimit",
    //"umask", depends on force_umask, see setup_seccomp
    //"uname", restrictions applied, see setup_seccomp
    "unshare",
    //"uselib",
    //"userfaultfd",
    "ustat",
    //"vfork", subject to fork protection
    //"vhangup",
    "vmsplice",
    //vserver
    "wait4",
    "waitid",
    //"write", crypt sandboxing
    //"writev", ditto
];

/// List of allowlisted ioctl requests.
pub const ALLOWLIST_IOCTL: &[KeyValue] = &[
    // Generic ioctls
    ("FIOASYNC", 0x5452),
    ("FIOCLEX", 0x5451),
    ("FIONBIO", 0x5421),
    ("FIONCLEX", 0x5450),
    ("FIONREAD", 0x541B),
    // Allow some font ioctls
    ("GIO_UNIMAP", 0x4B66),
    // Get and set terminal attributes
    ("TCGETA", 0x5405),
    ("TCGETS", 0x5401),
    ("TCSETA", 0x5406),
    ("TCSETAF", 0x5408),
    ("TCSETAW", 0x5407),
    ("TCSETS", 0x5402),
    ("TCSETSF", 0x5404),
    ("TCSETSW", 0x5403),
    // Locking the termios structure
    ("TIOCGLCKTRMIOS", 0x5456),
    ("TIOCSLCKTRMIOS", 0x5457),
    // Get and set window size
    ("TIOCGWINSZ", 0x5413),
    ("TIOCSWINSZ", 0x5414),
    // Sending a break
    ("TCSBRK", 0x5409),
    ("TCSBRKP", 0x5425),
    ("TIOCCBRK", 0x5428),
    ("TIOCSBRK", 0x5427),
    // Software flow control
    ("TCXONC", 0x540A),
    // Buffer count and flushing
    // Equal to FIONREAD: ("TIOCINQ", 0x541B),
    ("TCFLSH", 0x540B),
    ("TIOCOUTQ", 0x5411),
    // Controlling terminal.
    // SAFETY: These two ioctls are known-unsafe.
    // We disallow them.
    //ioctl_entry!(TIOCSTI),
    //ioctl_entry!(TIOCCONS),

    // Pty handling.
    ("TCGETX", 0x5432),
    ("TCSETX", 0x5433),
    ("TCSETXF", 0x5434),
    ("TCSETXW", 0x5435),
    ("TIOCGDEV", 0x80045432),
    ("TIOCGEXCL", 0x80045440),
    ("TIOCGPKT", 0x80045438),
    ("TIOCGPTLCK", 0x80045439),
    ("TIOCGPTN", 0x80045430),
    ("TIOCGPTPEER", 0x5441),
    ("TIOCSCTTY", 0x540E),
    ("TIOCSIG", 0x40045436),
    ("TIOCVHANGUP", 0x5437),
    // Process group and session ID
    ("TIOCGPGRP", 0x540F),
    ("TIOCGSID", 0x5429),
    ("TIOCSPGRP", 0x5410),
    // Exclusive mode
    ("TIOCEXCL", 0x540C),
    ("TIOCGEXCL", 0x80045440),
    ("TIOCNXCL", 0x540D),
    // Get/Set line discipline
    ("TIOCGETD", 0x5424),
    ("TIOCSETD", 0x5423),
    // Pseudoterminal ioctls
    ("TIOCGPKT", 0x80045438),
    ("TIOCGPTLCK", 0x80045439),
    ("TIOCGPTPEER", 0x5441),
    ("TIOCPKT", 0x5420),
    ("TIOCSPTLCK", 0x40045431),
    // SAFETY: Skip modem control
    // ioctls: TIOCMGET, TIOCMSET
    // et al.

    // Get file flags
    ("FS_IOC_GETFLAGS", 0x80086601),
    // Marking a line as local
    ("TIOCGSOFTCAR", 0x5419),
    ("TIOCSSOFTCAR", 0x541A),
    // Socket configuration controls
    ("SIOCGIFADDR", 0x8915),    // get PA address
    ("SIOCGIFBR", 0x8940),      // bridging support
    ("SIOCGIFBRDADDR", 0x8919), // get broadcast PA address
    ("SIOCGIFCONF", 0x8912),    // get iface list
    ("SIOCGIFCOUNT", 0x8938),   // get number of devices
    ("SIOCGIFDSTADDR", 0x8917), // get remote PA address
    ("SIOCGIFENCAP", 0x8925),   // get encapsulations
    ("SIOCGIFFLAGS", 0x8913),   // get flags
    ("SIOCGIFHWADDR", 0x8927),  // get hardware address
    ("SIOCGIFINDEX", 0x8933),   // name -> if_index mapping
    ("SIOCGIFMEM", 0x891f),     // get memory address (BSD)
    ("SIOCGIFMETRIC", 0x891d),  // get metric
    ("SIOCGIFMTU", 0x892),      // get MTU size
    ("SIOCGIFNAME", 0x8910),    // get iface name
    ("SIOCGIFNETMASK", 0x891b), // get network PA mask
    ("SIOCGIFPFLAGS", 0x8935),  // get extended flags set
    ("SIOCGIFSLAVE", 0x8929),   // driver slaving support
    ("SIOCGIFTXQLEN", 0x8942),  // get the tx queue length
    ("SIOCGMIIPHY", 0x8947),    // get address of MII PHY in use
    ("SIOCGMIIREG", 0x8948),    // Read MII PHY register
    ("SIOCGSKNS", 0x894C),      // get socket network namespace
];

/// List of perf ioctl requests.
pub const PERF_IOCTLS: &[KeyValue] = &[
    // ioctls that can be done on a perf event fd
    ("PERF_EVENT_IOC_DISABLE", 0x2401),
    ("PERF_EVENT_IOC_ENABLE", 0x2400),
    ("PERF_EVENT_IOC_ID", 0x80082407),
    ("PERF_EVENT_IOC_MODIFY_ATTRIBUTES", 0x4008240b),
    ("PERF_EVENT_IOC_PAUSE_OUTPUT", 0x40042409),
    ("PERF_EVENT_IOC_PERIOD", 0x40082404),
    ("PERF_EVENT_IOC_QUERY_BPF", 0xc008240a),
    ("PERF_EVENT_IOC_REFRESH", 0x2402),
    ("PERF_EVENT_IOC_RESET", 0x2403),
    ("PERF_EVENT_IOC_SET_BPF", 0x40042408),
    ("PERF_EVENT_IOC_SET_FILTER", 0x40082406),
    ("PERF_EVENT_IOC_SET_OUTPUT", 0x2405),
];

/// List of KVM ioctl requests
/// Read: https://www.kernel.org/doc/Documentation/virtual/kvm/api.txt
pub const KVM_IOCTLS: &[KeyValue] = &[
    ("KVM_GET_API_VERSION", 0x0000ae00),
    ("KVM_CREATE_VM", 0xae01),
    ("KVM_GET_VCPU_MMAP_SIZE", 0xae04),
    ("KVM_CREATE_VCPU", 0xae41),
    ("KVM_GET_DIRTY_LOG", 0x4010ae42),
    ("KVM_SET_TSS_ADDR", 0xae47),
    ("KVM_RUN", 0xae80),
    ("KVM_NMI", 0xae9a),
    ("KVM_CHECK_EXTENSION", 0xae03),
    ("KVM_GET_TSC_KHZ", 0xaea3),
    ("KVM_SET_TSC_KHZ", 0xaea2),
    ("KVM_INTERRUPT", 0x4004ae86),
    ("KVM_SET_MSRS", 0x4008ae89),
    ("KVM_SET_USER_MEMORY_REGION", 0x4020ae46),
    ("KVM_SET_REGS", 0x4090ae82),
    ("KVM_SET_SREGS", 0x4138ae84),
    ("KVM_GET_MSRS", 0xc008ae88),
    ("KVM_GET_REGS", 0x8090ae81),
    ("KVM_GET_SREGS", 0x8138ae83),
    ("KVM_GET_SUPPORTED_CPUID", 0xc008ae05),
    ("KVM_GET_EMULATED_CPUID", 0xc008ae09),
    ("KVM_SET_CPUID2", 0x4008ae90),
    ("KVM_SET_SIGNAL_MASK", 0x4004ae8b),
    ("KVM_GET_VCPU_EVENTS", 0x8040ae9f),
    ("KVM_SET_VCPU_EVENTS", 0x4040aea0),
    ("KVM_SET_DEVICE_ATTR", 0x4018aee1),
    ("KVM_SET_IDENTITY_MAP_ADDR", 0x4008ae48),
    ("KVM_CREATE_IRQCHIP", 0x0000ae60),
    ("KVM_IRQ_LINE", 0x4008ae61),
    ("KVM_REGISTER_COALESCED_MMIO", 0x4010ae67),
    ("KVM_UNREGISTER_COALESCED_MMIO", 0x4010ae68),
    ("KVM_SET_GSI_ROUTING", 0x4030ae6a),
    ("KVM_IRQFD", 0x4020ae76),
    ("KVM_IOEVENTFD", 0x4040ae79),
    ("KVM_GET_MP_STATE", 0x8004ae98),
    ("KVM_SET_MP_STATE", 0x4004ae99),
    ("KVM_SIGNAL_MSI", 0x4200aea5),
    ("KVM_SET_GUEST_DEBUG", 0x404ae09b),
    ("KVM_CREATE_DEVICE", 0xc00caee0),
    ("KVM_SET_DEVICE_ATTR", 0x4018aee1),
    ("KVM_GET_DEVICE_ATTR", 0x8018aee2),
    ("KVM_HAS_DEVICE_ATTR", 0x4018aee3),
];

/// List of arch-native KVM ioctl requests
#[cfg(any(target_arch = "arm", target_arch = "aarch64"))]
pub const KVM_IOCTLS_NATIVE: &[KeyValue] = &[
    ("_KVM_ENABLE_CAP", 0x4680aea3),
    ("_KVM_GET_ONE_REG", 0x4010aeab),
    ("_KVM_SET_ONE_REG", 0x4010aeac),
    ("_KVM_GET_REG_LIST", 0xc008aeb0),
    ("_KVM_ARM_TARGET_GENERIC_V8", 5),
    ("_KVM_ARM_PREFERRED_TARGET", 0x8020aeaf),
    ("_KVM_ARM_VCPU_INIT", 0x4020aeae),
    ("_KVM_ARM_VCPU_FINALIZE", 0x4040aec2),
    ("_KVM_ARM64_REGS_PSTATE", 0x6030000000100042),
    ("_KVM_ARM64_REGS_SP_EL1", 0x6030000000100044),
    ("_KVM_ARM64_REGS_R0", 0x6030000000100000),
    ("_KVM_ARM64_REGS_R1", 0x6030000000100002),
    ("_KVM_ARM64_REGS_R2", 0x6030000000100004),
    ("_KVM_ARM64_REGS_R3", 0x6030000000100006),
    ("_KVM_ARM64_REGS_R8", 0x6030000000100010),
    ("_KVM_ARM64_REGS_R18", 0x6030000000100024),
    ("_KVM_ARM64_REGS_PC", 0x6030000000100040),
    ("_KVM_ARM64_REGS_MAIR_EL1", 0x603000000013c510),
    ("_KVM_ARM64_REGS_TCR_EL1", 0x603000000013c102),
    ("_KVM_ARM64_REGS_TTBR0_EL1", 0x603000000013c100),
    ("_KVM_ARM64_REGS_TTBR1_EL1", 0x603000000013c101),
    ("_KVM_ARM64_REGS_SCTLR_EL1", 0x603000000013c080),
    ("_KVM_ARM64_REGS_CPACR_EL1", 0x603000000013c082),
    ("_KVM_ARM64_REGS_VBAR_EL1", 0x603000000013c600),
    ("_KVM_ARM64_REGS_TIMER_CNT", 0x603000000013df1a),
    ("_KVM_ARM64_REGS_CNTFRQ_EL0", 0x603000000013df00),
    ("_KVM_ARM64_REGS_MDSCR_EL1", 0x6030000000138012),
    ("_KVM_ARM64_REGS_CNTKCTL_EL1", 0x603000000013c708),
    ("_KVM_ARM64_REGS_TPIDR_EL1", 0x603000000013c684),
];

/// List of arch-native KVM ioctl requests
#[cfg(any(target_arch = "x86", target_arch = "x86_64"))]
pub const KVM_IOCTLS_NATIVE: &[KeyValue] = &[
    ("_KVM_VCPU_TSC_CTRL", 0x0),
    ("_KVM_VCPU_TSC_OFFSET", 0x0),
    ("_KVM_GET_MSR_INDEX_LIST", 0xc004ae02),
    ("_KVM_GET_MSR_FEATURE_INDEX_LIST", 0xc004ae0a),
    ("_KVM_CREATE_PIT2", 0x4040ae77),
    ("_KVM_GET_IRQCHIP", 0xc208ae62),
    ("_KVM_SET_IRQCHIP", 0x4208ae63),
    ("_KVM_SET_CLOCK", 0x4030ae7b),
    ("_KVM_GET_CLOCK", 0x8030ae7c),
    ("_KVM_GET_PIT2", 0x8070ae9f),
    ("_KVM_SET_PIT2", 0x4070aea0),
    ("_KVM_MEMORY_ENCRYPT_OP", 0xc008aeba),
    ("_KVM_MEMORY_ENCRYPT_REG_REGION", 0x8010aebb),
    ("_KVM_MEMORY_ENCRYPT_UNREG_REGION", 0x8010aebc),
    ("_KVM_TRANSLATE", 0xc018ae85),
    ("_KVM_GET_FPU", 0x81a0ae8c),
    ("_KVM_SET_FPU", 0x41a0ae8d),
    ("_KVM_GET_LAPIC", 0x8400ae8e),
    ("_KVM_SET_LAPIC", 0x4400ae8f),
    ("_KVM_SET_CPUID2", 0x4008ae90),
    ("_KVM_GET_CPUID2", 0xc008ae91),
    ("_KVM_GET_DEBUGREGS", 0x8080aea1),
    ("_KVM_SET_DEBUGREGS", 0x4080aea2),
    ("_KVM_GET_XSAVE", 0x9000aea4),
    ("_KVM_SET_XSAVE", 0x5000aea5),
    ("_KVM_GET_XCRS", 0x8188aea6),
    ("_KVM_SET_XCRS", 0x4188aea7),
    ("_KVM_KVMCLOCK_CTRL", 0xaead),
];

/// List of arch-native KVM ioctl requests
#[cfg(not(any(
    target_arch = "arm",
    target_arch = "x86",
    target_arch = "x86_64",
    target_arch = "aarch64"
)))]
pub const KVM_IOCTLS_NATIVE: &[KeyValue] = &[];
